//
//  ViewController.m
//  AgencyAppTest
//
//  Created by li carver on 1/17/13.
//  Copyright (c) 2013 Accela.Inc. All rights reserved.
//

#import "ViewController.h"

//Permesions for testing agency app
#define AGENCY_DEFAULT_PERMISSIONS @"get_records get_record create_record get_inspections get_record_documents get_document get_gis_settings create_record_document"

#define TAG_CREATE_RECORD 1
#define TAG_GET_RECORDS 2
#define TAG_GET_RECORD 21
#define TAG_GET_INSPECTIONS 3
#define TAG_DOWNLOAD_ATTACHMENT 4
#define TAG_DOWNLOAD_ATTACHMENT_LIST 41
#define TAG_UPLOAD_ATTACHMENT 5
#define TAG_GET_GIS_SETTINGS 6

@interface ViewController () {
    AppDelegate *appDelegate;
}

- (void)showPreviewWindowForImage:(UIImage *) image;
- (void)showError: (NSString *)title andError: (AMError *)error;
- (void)showResultWithTitle: (NSString *)title andMessage: (NSDictionary *) message;

@end

@implementation ViewController
@synthesize accelaMobile;

- (void)viewDidLoad
{
    [super viewDidLoad];
	
    //Make sure the AppID and AppSecret used in the following two lines exist on your mobile server and its type is agencyapp
    NSString *appId = @"634974410907971346";
    NSString *appSecret = @"9a5515e0482e44e6876ca49ef819b808";
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    appDelegate.accelaMobile = [[AccelaMobile alloc]initWithAppId:appId withAppSecret:appSecret andDelegate:self withAuthHost:nil withApiHost:nil];
    
    self.accelaMobile = [appDelegate accelaMobile];
    self.accelaMobile.sessionDelegate = self;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)clickAuth:(id)sender {	
    NSArray *permissions = [[NSString stringWithFormat:@"%@",AGENCY_DEFAULT_PERMISSIONS] componentsSeparatedByString:@" "];
    [self.accelaMobile authorize:permissions];
}


- (IBAction)clickAuth2:(id)sender {
    NSArray *permissions = [[NSString stringWithFormat:@"%@",AGENCY_DEFAULT_PERMISSIONS] componentsSeparatedByString:@" "];
    [self.accelaMobile authorize2:permissions];
}

- (IBAction)clickGetRecords:(id)sender {
    NSDictionary *urlParams = [NSDictionary dictionaryWithObjectsAndKeys:
                               @"0", @"offset",
                               @"10", @"limit",
                               nil];
    
    AMRequest *request = [self.accelaMobile requestWithPath: @"/v3/records/"
                                                  urlParams: urlParams
                                            requestDelegate: self];
    request.tag = TAG_GET_RECORDS;
    [request setAmOwnerView:self.view];
}

- (IBAction)clickGetRecordsSync:(id)sender {
    NSDictionary *urlParams = [NSDictionary dictionaryWithObjectsAndKeys:
                               @"0", @"offset",
                               @"10", @"limit",
                               nil];
    
	
	NSDictionary *result = [self.accelaMobile fetchWithPath:@"/v3/records/"  urlParams:urlParams error:^(AMError *error) {
		[self showError:@"getRecordSync" andError:error];
	}];
	
	if (result) {
		[self showResultWithTitle:@"getRecordSync" andMessage:result];
	}
}

- (IBAction)clickGetSpecificRecord:(id)sender {
    AMRequest *request = [self.accelaMobile requestWithPath:@"/v3/records/12CAP-00000-00001/"
                                                  urlParams:nil
                                                 httpMethod:GET
                                                       data:nil
                                            requestDelegate:self];
    request.tag = TAG_GET_RECORD;
    [request setAmOwnerView:self.view];
}

- (IBAction)clickCreateRecord:(id)sender {
    
    AMParameter *data = [[AMParameter alloc] initWithKey:@"createRecord"];
    // Create a record with description
    
	data.body = [NSDictionary dictionaryWithObjectsAndKeys:
                 [NSDictionary dictionaryWithObject:@"ServiceRequest-Graffiti-Graffiti-NA" forKey:@"id"], @"type",
                 @"A graffiti on the wall", @"description",
                 nil];
    AMRequest *request = [self.accelaMobile requestWithPath:@"/v3/records/"
                                                  urlParams:nil
                                                 httpMethod:POST
                                                       data:[data toData]
                                            requestDelegate:self];
    request.tag = TAG_CREATE_RECORD;
    [request setAmOwnerView:self.view];
}

- (IBAction)clickGetGISSetting:(id)sender {
    NSDictionary *urlParams = [NSDictionary dictionaryWithObjectsAndKeys:
                               self.accelaMobile.amClientId, @"client_id",
                               self.accelaMobile.amClientSecret,@"client_secret",
                               nil];
	
	AMRequest *request = [self.accelaMobile requestWithPath: @"/v3/GISSettings"
                                                  urlParams: urlParams
                                            requestDelegate: self];
    request.tag = TAG_GET_GIS_SETTINGS;
    [request setAmOwnerView:self.view];
}

- (IBAction)clickGetInspections:(id)sender {
    NSDictionary *urlParams = [NSDictionary dictionaryWithObjectsAndKeys:
                               @"0", @"offset",
                               @"10", @"limit",
                               nil];
    
    AMRequest *request = [self.accelaMobile requestWithPath: @"/v3/inspections/"
                                                  urlParams: urlParams
                                            requestDelegate: self];
    request.tag = TAG_GET_INSPECTIONS;
    [request setAmOwnerView:self.view];
}

- (IBAction)clickDownloadAttachmentList:(id)sender {
    NSDictionary *urlParams = [NSDictionary dictionaryWithObjectsAndKeys:
                               @"0", @"offset",
                               @"100", @"limit",
                               nil];
    
    AMRequest *request = [self.accelaMobile requestWithPath: @"/v3/records/13CAP-00000-00001/documents/"
                                                  urlParams: urlParams
                                            requestDelegate: self];
    request.tag = TAG_DOWNLOAD_ATTACHMENT_LIST;
    [request setAmOwnerView:self.view];
}

- (IBAction)clickDownloadAttachmentWithProgress:(id)sender {
    NSString *documentPath = [NSHomeDirectory() stringByAppendingString:@"/Documents"];
    
    NSString *localPath = [NSString stringWithFormat:@"%@/AccelaAnalytics.png",documentPath];
    
    NSString *url = @"/v3/document/13CAP-00000-00001-1681092/";
    
    AMRequest *downloadRequest = [self.accelaMobile downloadAttachment:url urlParams:nil localPath:localPath requestDelegate:self];
    
	//Show progress view if setting the property
    downloadRequest.amOwnerView = self.view;
    
    downloadRequest.tag = TAG_DOWNLOAD_ATTACHMENT;
}

- (IBAction)clickUploadAttachment:(id)sender {
    NSString *localPath = [[NSBundle mainBundle] pathForResource:@"AccelaAnalytics" ofType:@"png"];
	
    NSString *fileName = @"AccelaAnalytics.png";
    
    BOOL fileExists = [[NSFileManager defaultManager] fileExistsAtPath:localPath];
    
    if (fileExists) {
        
        NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
                              fileName, @"fileName",
                              localPath, @"localPath",
                              nil];
        
        NSString *url = @"/v3/records/12CAP-00000-00001/document/";
        
        [self.accelaMobile uploadAttachment:url urlParams:nil attachment:dict requestDelegate:self].tag = TAG_UPLOAD_ATTACHMENT;
    }else{
		UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:[NSString stringWithFormat:@"%@ doesn't exist.",localPath] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
		[alert show];
	}
}

- (IBAction)clickUploadAttachmentWithProgress:(id)sender {
    NSString *localPath = [[NSBundle mainBundle] pathForResource:@"AccelaAnalytics" ofType:@"png"];
	NSString *fileName = @"AccelaAnalytics.png";
    
    BOOL fileExists = [[NSFileManager defaultManager] fileExistsAtPath:localPath];
    
    if (fileExists) {
        NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
                              fileName, @"fileName",
                              localPath, @"localPath",
                              nil];
        
        NSString *url = @"/v3/records/12CAP-00000-00001/document/";
        
        AMRequest *uploadFile = [self.accelaMobile uploadAttachment:url urlParams:nil attachment:dict requestDelegate:self];
        
		//Show progress view if setting the property
        uploadFile.amOwnerView = self.view;
        
        uploadFile.tag = TAG_UPLOAD_ATTACHMENT;
    }else{
		UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:[NSString stringWithFormat:@"%@ doesn't exist.",localPath] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
		[alert show];
	}
}

- (IBAction)clickSignOut:(id)sender {
    [self.accelaMobile logout];
}

#pragma AMRequestDelegate

- (void)amRequest: (AMRequest *)request didReceiveResponse: (NSDictionary *)httpHeaders{
	NSLog(@"didReceiveResponse: %@",httpHeaders);
}

- (void)amRequest: (AMRequest *)request didFailWithError: (AMError *)error {
    switch (request.tag) {
		case TAG_CREATE_RECORD:
            [self showError:@"Failed to create record" andError:error];
			break;
		case TAG_DOWNLOAD_ATTACHMENT:
			[self showError:@"Failed to download attachment" andError:error];
			break;
		case TAG_GET_INSPECTIONS:
			[self showError:@"Failed to get Inspections" andError:error];
			break;
		case TAG_GET_RECORD:
            [self showError:@"Failed to get record list" andError:error];
			break;
		case TAG_UPLOAD_ATTACHMENT:
            [self showError:@"Failed to upload file" andError:error];
			break;
        case TAG_GET_RECORDS:
            [self showError:@"Failed to get specific record result" andError:error];
            break;
        case TAG_DOWNLOAD_ATTACHMENT_LIST:
            [self showError:@"Failed to download attachment list" andError:error];
            break;
		case TAG_GET_GIS_SETTINGS:
			[self showError:@"Failed to get gis settings" andError:error];
			break;
		default:
			break;
	}
}

- (void)amRequest: (AMRequest *)request didLoad: (id)result {
	switch (request.tag) {
		case TAG_CREATE_RECORD:
            [self showResultWithTitle:@"Create Record Result" andMessage:result];
			break;
		case TAG_DOWNLOAD_ATTACHMENT:
        {
			UIImage *image = [UIImage imageWithContentsOfFile:[request amDownloadDestinationPath]];
			[self showPreviewWindowForImage:image];
        }
			break;
		case TAG_GET_INSPECTIONS:
			[self showResultWithTitle:@"Get Inspections" andMessage:result];
			break;
		case TAG_GET_RECORDS:
            [self showResultWithTitle:@"Get Record List Result" andMessage:result];
			break;
		case TAG_UPLOAD_ATTACHMENT:
            [self showResultWithTitle:@"Upload File Result" andMessage:result];
			break;
        case TAG_GET_RECORD:
            [self showResultWithTitle:@"Get Specific Record Result" andMessage:result];
            break;
        case TAG_DOWNLOAD_ATTACHMENT_LIST:
            [self showResultWithTitle:@"Get Record Attachment List Result" andMessage:result];
            break;
		case TAG_GET_GIS_SETTINGS:
			[self showResultWithTitle:@"Get GISSetting without login" andMessage:result];
			break;
		default:
			break;
	}
}

#pragma AMSessionDelegate

-(void)amDidLogin:(int)grantType {
    NSLog(@"%@",@"amDidLogin");
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Login successfully!" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
	[alert show];
}

-(void)amDidCancelLogin {
    NSLog(@"%@",@"amDidCancelLogin");
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"You cancel login." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
	[alert show];
}

-(void)amDidLoginFailure:(AMError *)error {
	NSLog(@"%@",@"amDidLoginFailure");
    [self showError:@"Failed to get a token." andError:error];
}

-(BOOL)amDidSessionInvalid:(AMError *)error {
	NSLog(@"%@",@"amDidSessionInvalid");
    [self showError:@"Session Invalid" andError:error];
	return YES;
}

-(void)amDidLogout {
	NSLog(@"%@",@"amDidLogout");
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"You have signed out!" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
	[alert show];
}

/**
 * Show error message
 */
- (void)showError: (NSString *)title andError: (AMError *)error {
	NSString *message = [NSString stringWithFormat:@"Code: %d\nTraceId: %@\nDetail: %@", error.code, error.traceId, error.errorMessage];
    UIAlertView *popupMessage = [[UIAlertView alloc] initWithTitle:title
                                                           message:message
                                                          delegate:nil
                                                 cancelButtonTitle:@"OK"
                                                 otherButtonTitles: nil];
    
    [popupMessage show];
}

//Show the result
- (void)showResultWithTitle: (NSString *)title andMessage: (NSDictionary *) message {
    UIAlertView *popupMessage = [[UIAlertView alloc] initWithTitle:title
                                                           message:[NSString stringWithFormat:@"%@",message]
                                                          delegate:nil
                                                 cancelButtonTitle:@"OK"
                                                 otherButtonTitles: nil];
    
    [popupMessage show];
}

- (void)showPreviewWindowForImage:(UIImage *) image {
    
	UIViewController *imageViewController = [[UIViewController alloc]init];
	
	// Create tool bar with Cancel button
	UIBarButtonItem *backButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
                                                                                    target:self
                                                                                    action:@selector(closeImagePreviewWindow:)];
	
	UIBarButtonItem *flexibleSpaceItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace
                                                                                       target:nil
                                                                                       action:nil];
	UIToolbar *toolBar = [[UIToolbar alloc] init];
	toolBar.frame = CGRectMake(0.0, 0.0, self.view.frame.size.width,44.0);
	toolBar.barStyle = UIBarStyleBlackOpaque;
	toolBar.items = [NSArray arrayWithObjects:backButtonItem, flexibleSpaceItem, nil];
	
	// Create image view
	UIImageView *imageView = [[UIImageView alloc] initWithImage:image];
	imageView.frame = CGRectMake(0.0, 44.0, self.view.frame.size.width, self.view.frame.size.height - 44.0);
	
	// Add tool bar and iamge view to the pop-up view window
	[imageViewController.view addSubview:toolBar];
	[imageViewController.view addSubview:imageView];
	
	// Pop up the window
	[self presentModalViewController:imageViewController animated:YES];
}

//Close the pop-up image preview window
- (void)closeImagePreviewWindow:(id)sender {
    
	[self dismissModalViewControllerAnimated:YES];
}

@end