//
//  ViewController.m
//  CitizenAppTest
//
//  Created by li carver on 1/17/13.
//  Copyright (c) 2013 Accela.Inc. All rights reserved.
//

#import "ViewController.h"

//Permesions for testing citizen app
#define CITIZEN_DEFAULT_PERMISSIONS @"get_records get_record create_record search_records geocode_search_records"

#define TAG_GET_RECORDS 101
#define TAG_CREATE_RECORD 102
#define TAG_SEARCH_RECORD 103

@interface ViewController () {
    AppDelegate *appDelegate;
}

- (void)showError: (NSString *)title andError: (AMError *)error;
- (void)showResultWithTitle: (NSString *)title andMessage: (NSDictionary *) message;

@end

@implementation ViewController
@synthesize accelaMobile;

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    
    //Make sure the AppID and AppSecret used in the following two lines exist on your mobile server and its type is citizenapp
    NSString *appId = @"634974411260936557";
    NSString *appSecret = @"f881f3340dad4d028bba8d8f2c6b2fa6";
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    appDelegate.accelaMobile = [[AccelaMobile alloc]initWithAppId:appId withAppSecret:appSecret andDelegate:self];
    
    self.accelaMobile = [appDelegate accelaMobile];
    self.accelaMobile.envType = ENVIRONMENT_PROD;
	
	NSMutableDictionary *custHttpHeaders = [[NSMutableDictionary alloc] init];
	[custHttpHeaders setValue:@"islandton" forKey:@"x-accela-agency"];
	self.accelaMobile.httpHeaders = custHttpHeaders;
	
    self.accelaMobile.sessionDelegate = self;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)clickAuth:(id)sender {
    NSArray *permissions = [[NSString stringWithFormat:@"%@",CITIZEN_DEFAULT_PERMISSIONS] componentsSeparatedByString:@" "];
    [self.accelaMobile authorize:permissions withAgency:@"ISLANDTON"];
}

- (IBAction)clickAuth2:(id)sender {
    NSArray *permissions = [[NSString stringWithFormat:@"%@",CITIZEN_DEFAULT_PERMISSIONS] componentsSeparatedByString:@" "];
    [self.accelaMobile authorize2:permissions withAgency:@"ISLANDTON"];
}

- (IBAction)clickGetRecords:(id)sender {
    NSDictionary *urlParams = [NSDictionary dictionaryWithObjectsAndKeys:
                               @"0", @"offset",
                               @"10", @"limit",
                               nil];
    
    AMRequest *request = [self.accelaMobile requestWithPath: @"/v3/records"
                                                  urlParams: urlParams
                                            requestDelegate: self];
	
	request.tag = TAG_GET_RECORDS;
	request.amOwnerView = self.view;
}

- (IBAction)clickCreateRecord:(id)sender {
    AMParameter *data = [[AMParameter alloc] initWithKey:@"createRecord"];
    
    // Create a record with description
	data.body = [NSDictionary dictionaryWithObjectsAndKeys:
                 [NSDictionary dictionaryWithObject:@"ServiceRequest-Graffiti-Graffiti-NA" forKey:@"id"], @"type",
                 @"A graffiti on the wall", @"description",
                 nil];
    
    AMRequest *request = [self.accelaMobile requestWithPath:@"/v3/records/"
                                                  urlParams:nil
                                                 httpMethod:POST
                                                       data:[data toData]
                                            requestDelegate:self];
	request.tag = TAG_CREATE_RECORD;
	request.amOwnerView = self.view;
}

- (IBAction)clickSearchRecords:(id)sender {
    NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
                          @"1", @"offset",
                          @"20", @"limit",
                          @"Addresses", @"expand",
                          nil];
    AMRequest *recordsRequest = [self.accelaMobile requestWithPath:@"/v3/Search/Records" urlParams:dict httpMethod:POST data:nil requestDelegate:self];
    recordsRequest.tag= TAG_SEARCH_RECORD;
    recordsRequest.amOwnerView = self.view;
}

- (IBAction)clickSignOut:(id)sender {
    [self.accelaMobile logout];
}

#pragma AMRequestDelegate

- (void)amRequest: (AMRequest *)request didReceiveResponse: (NSDictionary *)httpHeaders{
	NSLog(@"didReceiveResponse: %@",httpHeaders);
}

- (void)amRequest: (AMRequest *)request didFailWithError: (AMError *)error {
    switch (request.tag) {
		case TAG_CREATE_RECORD:
            [self showError:@"Failed to create record" andError:error];
			break;
        case TAG_GET_RECORDS:
            [self showError:@"Failed to get specific record result" andError:error];
            break;
        case TAG_SEARCH_RECORD:
            [self showError:@"Failed to search record" andError:error];
            break;
		default:
			break;
	}
}

- (void)amRequest: (AMRequest *)request didLoad: (id)result {
	switch (request.tag) {
		case TAG_CREATE_RECORD:
            [self showResultWithTitle:@"Create Record Result" andMessage:result];
			break;
		case TAG_GET_RECORDS:
            [self showResultWithTitle:@"Get Record List Result" andMessage:result];
			break;
        case TAG_SEARCH_RECORD:
            [self showResultWithTitle:@"Search Record Result" andMessage:result];
            break;
		default:
			break;
	}
}

#pragma AMSessionDelegate

-(void)amDidLogin {
    NSLog(@"%@",@"amDidLogin");
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Login successfully!" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
	[alert show];
}

-(void)amDidCancelLogin {
    NSLog(@"%@",@"amDidCancelLogin");
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"You cancel login." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
	[alert show];
}

-(void)amDidLoginFailure:(AMError *)error {
	NSLog(@"%@",@"amDidLoginFailure");
    [self showError:@"Failed to get a token." andError:error];
}

-(BOOL)amDidSessionInvalid:(AMError *)error {
	NSLog(@"%@",@"amDidSessionInvalid");
    [self showError:@"Session Invalid" andError:error];
	return YES;
}

-(void)amDidLogout {
	NSLog(@"%@",@"amDidLogout");
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"You have signed out!" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
	[alert show];
}

/**
 * Show error message
 */
- (void)showError: (NSString *)title andError: (AMError *)error {
	NSString *message = [NSString stringWithFormat:@"Code: %d\nTraceId: %@\nDetail: %@", error.code, error.traceId, error.errorMessage];
    UIAlertView *popupMessage = [[UIAlertView alloc] initWithTitle:title
                                                           message:message
                                                          delegate:nil
                                                 cancelButtonTitle:@"OK"
                                                 otherButtonTitles: nil];
    
    [popupMessage show];
}

//Show the result
- (void)showResultWithTitle: (NSString *)title andMessage: (NSDictionary *) message {
    UIAlertView *popupMessage = [[UIAlertView alloc] initWithTitle:title
                                                           message:[NSString stringWithFormat:@"%@",message]
                                                          delegate:nil
                                                 cancelButtonTitle:@"OK"
                                                 otherButtonTitles: nil];
    
    [popupMessage show];
}

@end
